﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using HtmlAgilityPack;
using System.Text.Json;

namespace searchG
{
    public class SearchResult
    {
        public string Name { get; set; }
        public string Link { get; set; }
        public string Description { get; set; }
    }
    
    class Program
    {
        static void Main(string[] args)
        {
            Console.WriteLine("Starting HTTP server...");
            HttpListener listener = new HttpListener();
            listener.Prefixes.Add("http://localhost:50000/");
            listener.Start();
            Console.WriteLine("Server started on http://localhost:50000/");

            while (true)
            {
                HttpListenerContext context = listener.GetContext(); // Changed to blocking call for .NET Framework compatibility
                HandleRequest(context);
            }
        }

        static async void HandleRequest(HttpListenerContext context) // async void for event handlers
        {
            try
            {
                var request = context.Request;
                var response = context.Response;
                
                response.Headers.Add("Access-Control-Allow-Origin", "*");
                response.Headers.Add("Access-Control-Allow-Methods", "GET, OPTIONS");
                response.Headers.Add("Access-Control-Allow-Headers", "Content-Type");

                if (request.HttpMethod == "OPTIONS")
                {
                    response.StatusCode = 200;
                    return;
                }

                if (request.HttpMethod != "GET")
                {
                    response.StatusCode = 405;
                    return;
                }

                var queryParams = request.QueryString;
                string searchQuery = queryParams["query"];
                string pageStr = queryParams["page"] ?? "1";

                if (string.IsNullOrEmpty(searchQuery))
                {
                    response.StatusCode = 400;
                    await WriteResponse(response, "Missing required query parameter");
                    return;
                }

                if (!int.TryParse(pageStr, out int page) || page < 1)
                {
                    response.StatusCode = 400;
                    await WriteResponse(response, "Invalid pages parameter");
                    return;
                }

                var results = await ScrapeDuckDuckGo(searchQuery, page);

                response.StatusCode = 200;
                response.ContentType = "application/json";
                await WriteResponse(response, JsonSerializer.Serialize(results));
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: " + ex.Message);
            }
        }

        static async Task WriteResponse(HttpListenerResponse response, string message)
        {
            byte[] buffer = System.Text.Encoding.UTF8.GetBytes(message);
            response.ContentLength64 = buffer.Length;
            using (var output = response.OutputStream)
            {
                await output.WriteAsync(buffer, 0, buffer.Length);
            }
        }

        static async Task<List<SearchResult>> ScrapeDuckDuckGo(string query, int page)
        {
            using (var client = new HttpClient())
            {
                client.DefaultRequestHeaders.Add("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:133.0) Gecko/20100101 Firefox/133.0");
                client.DefaultRequestHeaders.Add("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
                client.DefaultRequestHeaders.Add("Accept-Language", "en-US,en;q=0.5");
                client.DefaultRequestHeaders.Add("Referer", "https://lite.duckduckgo.com/");
                client.DefaultRequestHeaders.Add("Origin", "https://lite.duckduckgo.com");
                client.DefaultRequestHeaders.Add("Connection", "keep-alive");
                client.DefaultRequestHeaders.Add("Upgrade-Insecure-Requests", "1");
                client.DefaultRequestHeaders.Add("Sec-Fetch-Dest", "document");
                client.DefaultRequestHeaders.Add("Sec-Fetch-Mode", "navigate");
                client.DefaultRequestHeaders.Add("Sec-Fetch-Site", "same-origin");
                client.DefaultRequestHeaders.Add("Sec-Fetch-User", "?1");
                client.DefaultRequestHeaders.Add("Priority", "u=0, i");
                client.DefaultRequestHeaders.Add("TE", "trailers");

                var formData = GenerateFormData(query, page);

                var response = await client.PostAsync("https://lite.duckduckgo.com/lite/", new FormUrlEncodedContent(formData));

                string html = await response.Content.ReadAsStringAsync();
                
                var doc = new HtmlDocument();
                doc.LoadHtml(html);

                var searchResults = new List<SearchResult>();

                var rows = doc.DocumentNode.SelectNodes("//tr");

                Console.WriteLine($"Found {rows.Count} rows");

                for (int i = 0; i < rows.Count; i++)
                {
                    if (i + 2 < rows.Count)
                    {
                        var nameNode = rows[i].SelectSingleNode(".//a");
                        var linkNode = rows[i + 2].SelectSingleNode(".//span[@class='link-text']");
                        var descriptionNode = rows[i + 1].SelectSingleNode(".//td[@class='result-snippet']");

                        if (nameNode != null && linkNode != null && descriptionNode != null)
                        {
                            var result = new SearchResult
                            {
                                Name = nameNode.InnerText.Trim(),
                                Link = nameNode.GetAttributeValue("href", ""),
                                Description = descriptionNode.InnerText.Trim()
                            };

                            searchResults.Add(result);
                        }
                    }
                    else
                    {
                        Console.WriteLine("Not enough rows for a complete result.");
                    }
                }

                return searchResults;
            }
        }

        static Dictionary<string, string> GenerateFormData(string query, int page)
        {
            var formData = new Dictionary<string, string>();
            
            formData.Add("q", query);

            switch (page)
            {
                case 2:
                    formData.Add("s", "10");
                    formData.Add("dc", "11");
                    break;
                case 3:
                    formData.Add("s", "25");
                    formData.Add("dc", "26");
                    break;
                case 4:
                    formData.Add("s", "40");
                    formData.Add("dc", "41");
                    break;
                case 5:
                    formData.Add("s", "55");
                    formData.Add("dc", "56");
                    break;
                case 6:
                    formData.Add("s", "70");
                    formData.Add("dc", "71");
                    break;
                case 7:
                    formData.Add("s", "85");
                    formData.Add("dc", "86");
                    break;
                case 8:
                    formData.Add("s", "100");
                    formData.Add("dc", "101");
                    break;
                case 9:
                    formData.Add("s", "115");
                    formData.Add("dc", "116");
                    break;
                case 10:
                    formData.Add("s", "130");
                    formData.Add("dc", "131");
                    break;
            }

            return formData;
        }
    }
}
